#!/usr/bin/env python
#-*- coding:utf-8 -*-
#
# Author:  Enrico Tröger
# License: BSD
#
# Script to report Logstash parse failures based on various tags on events.
# The report is printed to standard output, so the script is perfectly suitable
# as a standalone tool or to be used as a cronjob.
# Requires Python 2.x, elasticsearch and certifi.
# Setup: create a virtualenv and run: pip install elasticsearch certifi
# and adjust the following configuration settings as needed.

from elasticsearch import Elasticsearch
import certifi

ES_HOST = 'http://127.0.0.1:9200/'
#ES_HOST = 'https://api_user:verysecret@logs.example.com:9200/'
ES_INDEX = 'logstash-*'
ES_TIMEOUT = 60
# interval for searching parse failures, e.g. last seven days,
# see ElasticSearch docs for valid patterns
TIME_INTERVAL = '7d'
MAX_RESULTS = 30
# Short link to a corresponding Saved Search / Dashboard in Kibana
KIBANA_URL = 'https://logs.example.com/kibana4/goto/d03d05d31383b5491a51907136b99908'
# list of tags which indicate parse errors, adjust as needed
PARSE_FAILURE_TAGS = [
    '_grokparsefailure',
    '_jsonparsefailure',
    '_log_level_normalization_failed']


########################################################################
class LogstashParseFailureReporter(object):
    """"""

    #----------------------------------------------------------------------
    def __init__(self):
        self._es_client = None
        self._index = None
        self._timestamp = None
        self._site = None
        self._result = None

    #----------------------------------------------------------------------
    def report(self):
        self._setup_elastic_search_client()
        self._fetch_parse_failures()
        self._print_report()

    #----------------------------------------------------------------------
    def _setup_elastic_search_client(self):
        if ES_HOST.startswith('https'):
            self._es_client = Elasticsearch(
                hosts=[ES_HOST],
                use_ssl=True,
                verify_certs=True,
                ca_certs=certifi.where(),
                timeout=ES_TIMEOUT)
        else:
            self._es_client = Elasticsearch(
                hosts=[ES_HOST],
                timeout=ES_TIMEOUT)

    #----------------------------------------------------------------------
    def _fetch_parse_failures(self):
        request = {
            'from': 0,
            'query': {
                'filtered': {
                    'filter': {
                        'terms': {
                            'tags': PARSE_FAILURE_TAGS
                        }
                    }
                }
            },
            'filter':
            {
                'range': {
                    '@timestamp': {
                        'gt': 'now-{time_interval}'.format(time_interval=TIME_INTERVAL)
                    }
                }
            },
            'size': MAX_RESULTS,
            'sort': [
                {
                    '@timestamp': {'order': 'desc'}
                },
            ]
        }
        self._result = self._es_client.search(index=ES_INDEX, body=request)

    #----------------------------------------------------------------------
    def _print_report(self):
        hits = self._result['hits']['total']['value']
        if hits > MAX_RESULTS:
            print u'*There are more results than shown here* ({})!'.format(hits)
            print

        for hit in self._result['hits']['hits']:
            _type = hit['_type']
            _id = hit['_id']
            index = hit['_index']
            timestamp = hit['_source']['@timestamp'][:19]
            message = hit['_source']['message']
            hit_id = u'{}/{}/{}'.format(index, _type, _id)
            print u'{:62} {:20} {:.110} '.format(
                hit_id,
                timestamp,
                message)

        print
        print u'For details check the parse failures in Kibana at {}!'.format(KIBANA_URL)


#----------------------------------------------------------------------
def main():
    reporter = LogstashParseFailureReporter()
    reporter.report()


if __name__ == '__main__':
    main()
